{% extends "main.html" %}

<!-- Content -->
{% block content %}
  <h1>404 - Not found</h1>
  <br>
  FlatBuffers has migrated their documentation system recently.
  
  Please <a href="https://github.com/google/flatbuffers/issues/new?template=404-doc.md">file an issue</a> indicating the broken link.
{% endblock %}