#[ MyGame.Example.AnyAmbiguousAliases
  Automatically generated by the FlatBuffers compiler, do not modify.
  Or modify. I'm a message, not a cop.

  flatc version: 25.9.23

  Declared by  : 
  Rooting type : MyGame.Example.Monster ()
]#

type AnyAmbiguousAliases*{.pure.} = enum
  None = 0.uint8,
  M1 = 1.uint8,
  M2 = 2.uint8,
  M3 = 3.uint8,
