// automatically generated by the FlatBuffers compiler, do not modify

package MyGame.Example;

import com.google.flatbuffers.BaseVector;
import com.google.flatbuffers.BooleanVector;
import com.google.flatbuffers.ByteVector;
import com.google.flatbuffers.Constants;
import com.google.flatbuffers.DoubleVector;
import com.google.flatbuffers.FlatBufferBuilder;
import com.google.flatbuffers.FloatVector;
import com.google.flatbuffers.IntVector;
import com.google.flatbuffers.LongVector;
import com.google.flatbuffers.ShortVector;
import com.google.flatbuffers.StringVector;
import com.google.flatbuffers.Struct;
import com.google.flatbuffers.Table;
import com.google.flatbuffers.UnionVector;
import java.nio.ByteBuffer;
import java.nio.ByteOrder;

public class StructOfStructsOfStructsT {
  private MyGame.Example.StructOfStructsT a;

  public MyGame.Example.StructOfStructsT getA() { return a; }

  public void setA(MyGame.Example.StructOfStructsT a) { this.a = a; }


  public StructOfStructsOfStructsT() {
    this.a = new MyGame.Example.StructOfStructsT();
  }
}

