// automatically generated by the FlatBuffers compiler, do not modify

package NamespaceA.NamespaceB;

import com.google.flatbuffers.*;
import java.nio.*;
import java.util.*;

public class StructInNestedNST {
  private int a;
  private int b;

  public int getA() {
    return a;
  }

  public void setA(int a) {
    this.a = a;
  }

  public int getB() {
    return b;
  }

  public void setB(int b) {
    this.b = b;
  }

  public StructInNestedNST() {
    this.a = 0;
    this.b = 0;
  }
}
