// Automatically generated by the Flatbuffers compiler. Do not modify.
// @generated
pub mod keyword_test {
  use super::*;
  mod abc_generated;
  pub use self::abc_generated::*;
  mod public_generated;
  pub use self::public_generated::*;
  mod keywords_in_union_generated;
  pub use self::keywords_in_union_generated::*;
  mod keywords_in_table_generated;
  pub use self::keywords_in_table_generated::*;
  mod table_2_generated;
  pub use self::table_2_generated::*;
} // keyword_test
