-- automatically generated by the FlatBuffers compiler, do not modify

-- namespace: NamespaceB

local EnumInNestedNS = {
    A = 0,
    B = 1,
    C = 2,
}

return EnumInNestedNS -- return the module