// automatically generated by the FlatBuffers compiler, do not modify

package MyGame.Example;

import com.google.flatbuffers.BaseVector;
import com.google.flatbuffers.BooleanVector;
import com.google.flatbuffers.ByteVector;
import com.google.flatbuffers.Constants;
import com.google.flatbuffers.DoubleVector;
import com.google.flatbuffers.FlatBufferBuilder;
import com.google.flatbuffers.FloatVector;
import com.google.flatbuffers.IntVector;
import com.google.flatbuffers.LongVector;
import com.google.flatbuffers.ShortVector;
import com.google.flatbuffers.StringVector;
import com.google.flatbuffers.Struct;
import com.google.flatbuffers.Table;
import com.google.flatbuffers.UnionVector;
import java.nio.ByteBuffer;
import java.nio.ByteOrder;

public class TypeAliasesT {
  private byte i8;
  private int u8;
  private short i16;
  private int u16;
  private int i32;
  private long u32;
  private long i64;
  private long u64;
  private float f32;
  private double f64;
  private byte[] v8;
  private double[] vf64;

  public byte getI8() { return i8; }

  public void setI8(byte i8) { this.i8 = i8; }

  public int getU8() { return u8; }

  public void setU8(int u8) { this.u8 = u8; }

  public short getI16() { return i16; }

  public void setI16(short i16) { this.i16 = i16; }

  public int getU16() { return u16; }

  public void setU16(int u16) { this.u16 = u16; }

  public int getI32() { return i32; }

  public void setI32(int i32) { this.i32 = i32; }

  public long getU32() { return u32; }

  public void setU32(long u32) { this.u32 = u32; }

  public long getI64() { return i64; }

  public void setI64(long i64) { this.i64 = i64; }

  public long getU64() { return u64; }

  public void setU64(long u64) { this.u64 = u64; }

  public float getF32() { return f32; }

  public void setF32(float f32) { this.f32 = f32; }

  public double getF64() { return f64; }

  public void setF64(double f64) { this.f64 = f64; }

  public byte[] getV8() { return v8; }

  public void setV8(byte[] v8) { this.v8 = v8; }

  public double[] getVf64() { return vf64; }

  public void setVf64(double[] vf64) { this.vf64 = vf64; }


  public TypeAliasesT() {
    this.i8 = 0;
    this.u8 = 0;
    this.i16 = 0;
    this.u16 = 0;
    this.i32 = 0;
    this.u32 = 0L;
    this.i64 = 0L;
    this.u64 = 0L;
    this.f32 = 0.0f;
    this.f64 = 0.0;
    this.v8 = null;
    this.vf64 = null;
  }
}

