// automatically generated by the FlatBuffers compiler, do not modify
/* eslint-disable @typescript-eslint/no-unused-vars, @typescript-eslint/no-explicit-any,
 * @typescript-eslint/no-non-null-assertion */
export * as Example from './my-game/example.js';
export * as Example2 from './my-game/example2.js';
export {InParentNamespace, InParentNamespaceT} from './my-game/in-parent-namespace.js';
export * as OtherNameSpace from './my-game/other-name-space.js';
