// automatically generated by the FlatBuffers compiler, do not modify

/* eslint-disable @typescript-eslint/no-unused-vars, @typescript-eslint/no-explicit-any, @typescript-eslint/no-non-null-assertion */

export { Attacker, AttackerT } from './attacker.js';
export { BookReader, BookReaderT } from './book-reader.js';
export { Character } from './character.js';
export { FallingTub, FallingTubT } from './falling-tub.js';
export { Gadget } from './gadget.js';
export { HandFan, HandFanT } from './hand-fan.js';
export { Movie, MovieT } from './movie.js';
export { Rapunzel, RapunzelT } from './rapunzel.js';
