<?php
// automatically generated by the FlatBuffers compiler, do not modify

class Gadget
{
    const NONE = 0;
    const FallingTub = 1;
    const HandFan = 2;

    private static $names = array(
        Gadget::NONE=>"NONE",
        Gadget::FallingTub=>"FallingTub",
        Gadget::HandFan=>"HandFan",
    );

    public static function Name($e)
    {
        if (!isset(self::$names[$e])) {
            throw new \Exception();
        }
        return self::$names[$e];
    }
}
