// automatically generated by the FlatBuffers compiler, do not modify
/* eslint-disable @typescript-eslint/no-unused-vars, @typescript-eslint/no-explicit-any,
 * @typescript-eslint/no-non-null-assertion */
export {Ability, AbilityT} from './example/ability.js';
export {AnyAmbiguousAliases} from './example/any-ambiguous-aliases.js';
export {AnyUniqueAliases} from './example/any-unique-aliases.js';
export {Any} from './example/any.js';
export {Color} from './example/color.js';
export {LongEnum} from './example/long-enum.js';
export {Monster, MonsterT} from './example/monster.js';
export {Race} from './example/race.js';
export {Referrable, ReferrableT} from './example/referrable.js';
export {Stat, StatT} from './example/stat.js';
export {StructOfStructsOfStructs, StructOfStructsOfStructsT} from './example/struct-of-structs-of-structs.js';
export {StructOfStructs, StructOfStructsT} from './example/struct-of-structs.js';
export {TestSimpleTableWithEnum, TestSimpleTableWithEnumT} from './example/test-simple-table-with-enum.js';
export {Test, TestT} from './example/test.js';
export {TypeAliases, TypeAliasesT} from './example/type-aliases.js';
export {Vec3, Vec3T} from './example/vec3.js';
