// Code generated by the FlatBuffers compiler. DO NOT EDIT.

package Example

import "strconv"

type Race int8

const (
	RaceNone  Race = -1
	RaceHuman Race = 0
	RaceDwarf Race = 1
	RaceElf   Race = 2
)

var EnumNamesRace = map[Race]string{
	RaceNone:  "None",
	RaceHuman: "Human",
	RaceDwarf: "Dwarf",
	RaceElf:   "Elf",
}

var EnumValuesRace = map[string]Race{
	"None":  RaceNone,
	"Human": RaceHuman,
	"Dwarf": RaceDwarf,
	"Elf":   RaceElf,
}

func (v Race) String() string {
	if s, ok := EnumNamesRace[v]; ok {
		return s
	}
	return "Race(" + strconv.FormatInt(int64(v), 10) + ")"
}
