namespace KeywordTest;

enum ABC: int { void, where, stackalloc }

enum public: int { }

table KeywordsInTable {
  is: ABC = void;
  private: public;
  type: int;
  default: bool = false;
}

union KeywordsInUnion {
  static: KeywordsInTable,
  internal: KeywordsInTable,
}

table Table2 {
  type: KeywordsInUnion;
}
