namespace Swift.Tests;

table NanInfTable
{
  default_nan:double = nan;
  default_inf:double = inf;
  default_ninf:double = -inf;
  value_nan:double;
  value_inf:double;
  value_ninf:double;
  value:double;
}

root_type NanInfTable;
