// automatically generated by the FlatBuffers compiler, do not modify

package MyGame.Example;

import com.google.flatbuffers.BaseVector;
import com.google.flatbuffers.BooleanVector;
import com.google.flatbuffers.ByteVector;
import com.google.flatbuffers.Constants;
import com.google.flatbuffers.DoubleVector;
import com.google.flatbuffers.FlatBufferBuilder;
import com.google.flatbuffers.FloatVector;
import com.google.flatbuffers.IntVector;
import com.google.flatbuffers.LongVector;
import com.google.flatbuffers.ShortVector;
import com.google.flatbuffers.StringVector;
import com.google.flatbuffers.Struct;
import com.google.flatbuffers.Table;
import com.google.flatbuffers.UnionVector;
import java.nio.ByteBuffer;
import java.nio.ByteOrder;

public class StatT {
  private String id;
  private long val;
  private int count;

  public String getId() { return id; }

  public void setId(String id) { this.id = id; }

  public long getVal() { return val; }

  public void setVal(long val) { this.val = val; }

  public int getCount() { return count; }

  public void setCount(int count) { this.count = count; }


  public StatT() {
    this.id = null;
    this.val = 0L;
    this.count = 0;
  }
}

