-- automatically generated by the FlatBuffers compiler, do not modify

-- namespace: NamespaceB

local UnionInNestedNS = {
    NONE = 0,
    TableInNestedNS = 1,
}

return UnionInNestedNS -- return the module