// automatically generated by the FlatBuffers compiler, do not modify
/* eslint-disable @typescript-eslint/no-unused-vars, @typescript-eslint/no-explicit-any,
 * @typescript-eslint/no-non-null-assertion */
export {FromInclude} from './other-name-space/from-include.js';
export {TableB, TableBT} from './other-name-space/table-b.js';
export {Unused, UnusedT} from './other-name-space/unused.js';
