--[[ MyGame.Example.Race

  Automatically generated by the FlatBuffers compiler, do not modify.
  Or modify. I'm a message, not a cop.

  flatc version: 25.9.23

  Declared by  : //monster_test.fbs
  Rooting type : MyGame.Example.Monster (//monster_test.fbs)

--]]

local Race = {
  None = -1,
  Human = 0,
  Dwarf = 1,
  Elf = 2,
}

return Race