# automatically generated by the FlatBuffers compiler, do not modify

# namespace: reflection


class BaseType(object):
    None_ = 0
    UType = 1
    Bool = 2
    Byte = 3
    UByte = 4
    Short = 5
    UShort = 6
    Int = 7
    UInt = 8
    Long = 9
    ULong = 10
    Float = 11
    Double = 12
    String = 13
    Vector = 14
    Obj = 15
    Union = 16
    Array = 17
    MaxBaseType = 18
