export {A, AT} from './union-underlying-type/a.js';
export {ABC} from './union-underlying-type/abc.js';
export {B, BT} from './union-underlying-type/b.js';
export {C, CT} from './union-underlying-type/c.js';
export {D, DT} from './union-underlying-type/d.js';
