<?php
// automatically generated by the FlatBuffers compiler, do not modify

namespace MyGame\Example;

class LongEnum
{
    const LongOne = 2;
    const LongTwo = 4;
    const LongBig = 1099511627776;

    private static $names = array(
        LongEnum::LongOne=>"LongOne",
        LongEnum::LongTwo=>"LongTwo",
        LongEnum::LongBig=>"LongBig",
    );

    public static function Name($e)
    {
        if (!isset(self::$names[$e])) {
            throw new \Exception();
        }
        return self::$names[$e];
    }
}
