// automatically generated by the FlatBuffers compiler, do not modify
/* eslint-disable @typescript-eslint/no-unused-vars, @typescript-eslint/no-explicit-any,
 * @typescript-eslint/no-non-null-assertion */
export var Race;
(function(Race) {
Race[Race['None'] = -1] = 'None';
Race[Race['Human'] = 0] = 'Human';
Race[Race['Dwarf'] = 1] = 'Dwarf';
Race[Race['Elf'] = 2] = 'Elf';
})(Race || (Race = {}));
