#[ optional_scalars.OptionalByte
  Automatically generated by the FlatBuffers compiler, do not modify.
  Or modify. I'm a message, not a cop.

  flatc version: 25.9.23

  Declared by  : 
  Rooting type : optional_scalars.ScalarStuff ()
]#

type OptionalByte*{.pure.} = enum
  None = 0.int8,
  One = 1.int8,
  Two = 2.int8,
