export {Attacker, AttackerT} from './attacker.js';
export {BookReader, BookReaderT} from './book-reader.js';
export {Character} from './character.js';
export {FallingTub, FallingTubT} from './falling-tub.js';
export {Gadget} from './gadget.js';
export {HandFan, HandFanT} from './hand-fan.js';
export {Movie, MovieT} from './movie.js';
export {Rapunzel, RapunzelT} from './rapunzel.js';
