// automatically generated by the FlatBuffers compiler, do not modify

@Suppress("unused")
class Gadget private constructor() {
    companion object {
        const val NONE: UByte = 0u
        const val FallingTub: UByte = 1u
        const val HandFan: UByte = 2u
        val names : Array<String> = arrayOf("NONE", "FallingTub", "HandFan")
        fun name(e: Int) : String = names[e]
    }
}
