// automatically generated by the FlatBuffers compiler, do not modify

package MyGame.Example

@Suppress("unused")
class AnyAmbiguousAliases private constructor() {
    companion object {
        const val NONE: UByte = 0u
        const val M1: UByte = 1u
        const val M2: UByte = 2u
        const val M3: UByte = 3u
        val names : Array<String> = arrayOf("NONE", "M1", "M2", "M3")
        fun name(e: Int) : String = names[e]
    }
}
