// automatically generated by the FlatBuffers compiler, do not modify
/* eslint-disable @typescript-eslint/no-unused-vars, @typescript-eslint/no-explicit-any,
 * @typescript-eslint/no-non-null-assertion */
import * as flatbuffers from 'flatbuffers';
export class Attacker {
  constructor() {
    this.bb = null;
    this.bb_pos = 0;
  }
  __init(i, bb) {
    this.bb_pos = i;
    this.bb = bb;
    return this;
  }
  static getRootAsAttacker(bb, obj) {
    return (obj || new Attacker())
        .__init(bb.readInt32(bb.position()) + bb.position(), bb);
  }
  static getSizePrefixedRootAsAttacker(bb, obj) {
    bb.setPosition(bb.position() + flatbuffers.SIZE_PREFIX_LENGTH);
    return (obj || new Attacker())
        .__init(bb.readInt32(bb.position()) + bb.position(), bb);
  }
  swordAttackDamage() {
    const offset = this.bb.__offset(this.bb_pos, 4);
    return offset ? this.bb.readInt32(this.bb_pos + offset) : 0;
  }
  mutate_sword_attack_damage(value) {
    const offset = this.bb.__offset(this.bb_pos, 4);
    if (offset === 0) {
      return false;
    }
    this.bb.writeInt32(this.bb_pos + offset, value);
    return true;
  }
  static getFullyQualifiedName() {
    return 'Attacker';
  }
  static startAttacker(builder) {
    builder.startObject(1);
  }
  static addSwordAttackDamage(builder, swordAttackDamage) {
    builder.addFieldInt32(0, swordAttackDamage, 0);
  }
  static endAttacker(builder) {
    const offset = builder.endObject();
    return offset;
  }
  static createAttacker(builder, swordAttackDamage) {
    Attacker.startAttacker(builder);
    Attacker.addSwordAttackDamage(builder, swordAttackDamage);
    return Attacker.endAttacker(builder);
  }
  unpack() {
    return new AttackerT(this.swordAttackDamage());
  }
  unpackTo(_o) {
    _o.swordAttackDamage = this.swordAttackDamage();
  }
}
export class AttackerT {
  constructor(swordAttackDamage = 0) {
    this.swordAttackDamage = swordAttackDamage;
  }
  pack(builder) {
    return Attacker.createAttacker(builder, this.swordAttackDamage);
  }
}
