// automatically generated by the FlatBuffers compiler, do not modify

/* eslint-disable @typescript-eslint/no-unused-vars, @typescript-eslint/no-explicit-any, @typescript-eslint/no-non-null-assertion */

import * as flatbuffers from 'flatbuffers';



export class Attacker implements flatbuffers.IUnpackableObject<AttackerT> {
  bb: flatbuffers.ByteBuffer|null = null;
  bb_pos = 0;
  __init(i:number, bb:flatbuffers.ByteBuffer):Attacker {
  this.bb_pos = i;
  this.bb = bb;
  return this;
}

static getRootAsAttacker(bb:flatbuffers.ByteBuffer, obj?:Attacker):Attacker {
  return (obj || new Attacker()).__init(bb.readInt32(bb.position()) + bb.position(), bb);
}

static getSizePrefixedRootAsAttacker(bb:flatbuffers.ByteBuffer, obj?:Attacker):Attacker {
  bb.setPosition(bb.position() + flatbuffers.SIZE_PREFIX_LENGTH);
  return (obj || new Attacker()).__init(bb.readInt32(bb.position()) + bb.position(), bb);
}

swordAttackDamage():number {
  const offset = this.bb!.__offset(this.bb_pos, 4);
  return offset ? this.bb!.readInt32(this.bb_pos + offset) : 0;
}

mutate_sword_attack_damage(value:number):boolean {
  const offset = this.bb!.__offset(this.bb_pos, 4);

  if (offset === 0) {
    return false;
  }

  this.bb!.writeInt32(this.bb_pos + offset, value);
  return true;
}

static getFullyQualifiedName(): "Attacker" {
  return 'Attacker';
}

static startAttacker(builder:flatbuffers.Builder) {
  builder.startObject(1);
}

static addSwordAttackDamage(builder:flatbuffers.Builder, swordAttackDamage:number) {
  builder.addFieldInt32(0, swordAttackDamage, 0);
}

static endAttacker(builder:flatbuffers.Builder):flatbuffers.Offset {
  const offset = builder.endObject();
  return offset;
}

static createAttacker(builder:flatbuffers.Builder, swordAttackDamage:number):flatbuffers.Offset {
  Attacker.startAttacker(builder);
  Attacker.addSwordAttackDamage(builder, swordAttackDamage);
  return Attacker.endAttacker(builder);
}

unpack(): AttackerT {
  return new AttackerT(
    this.swordAttackDamage()
  );
}


unpackTo(_o: AttackerT): void {
  _o.swordAttackDamage = this.swordAttackDamage();
}
}

export class AttackerT implements flatbuffers.IGeneratedObject {
constructor(
  public swordAttackDamage: number = 0
){}


pack(builder:flatbuffers.Builder): flatbuffers.Offset {
  return Attacker.createAttacker(builder,
    this.swordAttackDamage
  );
}
}
