// automatically generated by the FlatBuffers compiler, do not modify
/* eslint-disable @typescript-eslint/no-unused-vars, @typescript-eslint/no-explicit-any,
 * @typescript-eslint/no-non-null-assertion */
export var FromInclude;
(function(FromInclude) {
FromInclude['IncludeVal'] = '0';
})(FromInclude || (FromInclude = {}));
