#[ MyGame.Example.Any
  Automatically generated by the FlatBuffers compiler, do not modify.
  Or modify. I'm a message, not a cop.

  flatc version: 25.9.23

  Declared by  : 
  Rooting type : MyGame.Example.Monster ()
]#

type Any*{.pure.} = enum
  None = 0.uint8,
  Monster = 1.uint8,
  TestSimpleTableWithEnum = 2.uint8,
  MyGameExample2Monster = 3.uint8,
