include 'test_dir/typescript_include.fbs';
include 'reflection/reflection.fbs';

// This file includes a variety of keywords meant to deliberately interfere
// with typescript keywords.
namespace typescript;

enum class: int {
  new,
  instanceof,
}

table Object {
  return:int;
  if:int;
  switch:int;
  enum:class;
  enum2:foobar.class;
  enum3:foobar.Abc;
  reflect:reflection.Schema;
}
