// automatically generated by the FlatBuffers compiler, do not modify

package NamespaceA.NamespaceB

import com.google.flatbuffers.FlatBufferBuilder
import com.google.flatbuffers.Struct
import java.nio.ByteBuffer

@Suppress("unused")
@ExperimentalUnsignedTypes
class StructInNestedNS : Struct() {

  fun __init(_i: Int, _bb: ByteBuffer) {
    __reset(_i, _bb)
  }

  fun __assign(_i: Int, _bb: ByteBuffer): StructInNestedNS {
    __init(_i, _bb)
    return this
  }

  val a: Int
    get() = bb.getInt(bb_pos + 0)

  fun mutateA(a: Int): ByteBuffer = bb.putInt(bb_pos + 0, a)

  val b: Int
    get() = bb.getInt(bb_pos + 4)

  fun mutateB(b: Int): ByteBuffer = bb.putInt(bb_pos + 4, b)

  companion object {
    fun createStructInNestedNS(builder: FlatBufferBuilder, a: Int, b: Int): Int {
      builder.prep(4, 8)
      builder.putInt(b)
      builder.putInt(a)
      return builder.offset()
    }
  }
}
