- [ ] I did **not** use any AI-assistance tools to help create this pull request.
- [x] I **did** use AI-assistance tools to *help* create this pull request.
- [x] I have read, understood and followed the project's AI_POLICY.md when creating code, documentation etc. for this pull request.

Submitted by: @{{ user }}
Date: {{ date }}
Related issue(s): #XXX
Branch: {{ user }}:{{ branch }}
