--[[ MyGame.OtherNameSpace.FromInclude

  Automatically generated by the FlatBuffers compiler, do not modify.
  Or modify. I'm a message, not a cop.

  flatc version: 25.9.23

  Declared by  : //include_test/sub/include_test2.fbs
  Rooting type : MyGame.Example.Monster (//monster_test.fbs)

--]]

local FromInclude = {
  IncludeVal = 0,
}

return FromInclude