namespace union_value_collsion;

table IntValue {
	value:int;
}
table Collide {
	collide: string (key);
	value: string;
}

union Value { IntValue }

union Other { IntValue }

// This table tests collsions of Unions and fields named value.
table Collision {
  some_value : Value;
  value : Other;
  collide : [Collision];
}

root_type Collision;