// automatically generated by the FlatBuffers compiler, do not modify

/* eslint-disable @typescript-eslint/no-unused-vars, @typescript-eslint/no-explicit-any, @typescript-eslint/no-non-null-assertion */

export * as foobar from './foobar.js';
export * as reflection from './reflection.js';
export * as typescript from './typescript.js';
