#[ Abc
  Automatically generated by the FlatBuffers compiler, do not modify.
  Or modify. I'm a message, not a cop.

  flatc version: 25.9.23

  Declared by  : 
]#

type Abc*{.pure.} = enum
  A = 0.int32,
  B = 1.int32,
  C = 2.int32,
