// Sample .proto file that we can not translate to the corresponding .fbs because it has used an id twice

option some_option = is_ignored;

package proto.test;

message ProtoMessage {
  optional sint32 e = 2;
  optional uint64 twice = 2;
}
