#[ MyGame.Example.LongEnum
  Automatically generated by the FlatBuffers compiler, do not modify.
  Or modify. I'm a message, not a cop.

  flatc version: 25.9.23

  Declared by  : 
  Rooting type : MyGame.Example.Monster ()
]#

type LongEnum*{.pure.} = enum
  LongOne = 2.uint64,
  LongTwo = 4.uint64,
  LongBig = 1099511627776.uint64,
