// Automatically generated by the Flatbuffers compiler. Do not modify.
// @generated
pub mod optional_scalars {
  use super::*;
  mod optional_byte_generated;
  pub use self::optional_byte_generated::*;
  mod scalar_stuff_generated;
  pub use self::scalar_stuff_generated::*;
} // optional_scalars
