// automatically generated by the FlatBuffers compiler, do not modify
export class Test {
  constructor() {
    this.bb = null;
    this.bb_pos = 0;
  }
  __init(i, bb) {
    this.bb_pos = i;
    this.bb = bb;
    return this;
  }
  a() {
    return this.bb.readInt16(this.bb_pos);
  }
  mutate_a(value) {
    this.bb.writeInt16(this.bb_pos + 0, value);
    return true;
  }
  b() {
    return this.bb.readInt8(this.bb_pos + 2);
  }
  mutate_b(value) {
    this.bb.writeInt8(this.bb_pos + 2, value);
    return true;
  }
  static getFullyQualifiedName() {
    return 'MyGame.Example.Test';
  }
  static sizeOf() {
    return 4;
  }
  static createTest(builder, a, b) {
    builder.prep(2, 4);
    builder.pad(1);
    builder.writeInt8(b);
    builder.writeInt16(a);
    return builder.offset();
  }
  unpack() {
    return new TestT(this.a(), this.b());
  }
  unpackTo(_o) {
    _o.a = this.a();
    _o.b = this.b();
  }
}
export class TestT {
  constructor(a = 0, b = 0) {
    this.a = a;
    this.b = b;
  }
  pack(builder) {
    return Test.createTest(builder, this.a, this.b);
  }
}
