// automatically generated by the FlatBuffers compiler, do not modify

@Suppress("unused")
class Character_ private constructor() {
    companion object {
        const val NONE: UByte = 0u
        const val MuLan: UByte = 1u
        const val Rapunzel: UByte = 2u
        const val Belle: UByte = 3u
        const val BookFan: UByte = 4u
        const val Other: UByte = 5u
        const val Unused: UByte = 6u
        val names : Array<String> = arrayOf("NONE", "MuLan", "Rapunzel", "Belle", "BookFan", "Other", "Unused")
        fun name(e: Int) : String = names[e]
    }
}
