// automatically generated by the FlatBuffers compiler, do not modify

package com.google.flatbuffers.reflection;

/**
 * New schema language features that are not supported by old code generators.
 */
@SuppressWarnings("unused")
public final class AdvancedFeatures {
  private AdvancedFeatures() { }
  public static final long AdvancedArrayFeatures = 1L;
  public static final long AdvancedUnionFeatures = 2L;
  public static final long OptionalScalars = 4L;
  public static final long DefaultVectorsAndStrings = 8L;
}

