// Automatically generated by the Flatbuffers compiler. Do not modify.
// @generated
pub mod my_game {
  use super::*;
  pub mod sample {
    use super::*;
    mod color_generated;
    pub use self::color_generated::*;
    mod equipment_generated;
    pub use self::equipment_generated::*;
    mod vec_3_generated;
    pub use self::vec_3_generated::*;
    mod monster_generated;
    pub use self::monster_generated::*;
    mod weapon_generated;
    pub use self::weapon_generated::*;
  } // sample
} // my_game
