#[ MyGame.OtherNameSpace.FromInclude
  Automatically generated by the FlatBuffers compiler, do not modify.
  Or modify. I'm a message, not a cop.

  flatc version: 25.9.23

  Declared by  : 
  Rooting type : MyGame.Example.Monster ()
]#

type FromInclude*{.pure.} = enum
  IncludeVal = 0.int64,
