// automatically generated by the FlatBuffers compiler, do not modify

package optional_scalars

@Suppress("unused")
class OptionalByte private constructor() {
    companion object {
        const val None: Byte = 0
        const val One: Byte = 1
        const val Two: Byte = 2
    }
}
