// automatically generated by the FlatBuffers compiler, do not modify

package MyGame.Example

@Suppress("unused")
class LongEnum private constructor() {
    companion object {
        const val LongOne: ULong = 2UL
        const val LongTwo: ULong = 4UL
        const val LongBig: ULong = 1099511627776UL
    }
}
