namespace Evolution.V1;

table TableA {
    a:float;
    b:int;
}

table TableB {
    a:int;
}

enum Enum : byte {
    King,
    Queen
}

union Union {
    TableA,
    TableB
}

struct Struct {
    a:int;
    b:double;
}

table Root {
    a:int;
    b:bool;
    c:Union;
    d:Enum;
    e:TableA;
    f:Struct;
    g:[int];
    h:[TableB];
    i:int = 1234;
    j:Union;
}

root_type Root;
