// automatically generated by the FlatBuffers compiler, do not modify

/* eslint-disable @typescript-eslint/no-unused-vars, @typescript-eslint/no-explicit-any, @typescript-eslint/no-non-null-assertion */

/**
 * Composite components of Monster color.
 */
export enum Color {
  Red = 1,

  /**
   * \brief color Green
   * Green is bit_flag with value (1u << 1)
   */
  Green = 2,

  /**
   * \brief color Blue (1u << 3)
   */
  Blue = 8
}
