--[[ MyGame.Example.LongEnum

  Automatically generated by the FlatBuffers compiler, do not modify.
  Or modify. I'm a message, not a cop.

  flatc version: 25.9.23

  Declared by  : //monster_test.fbs
  Rooting type : MyGame.Example.Monster (//monster_test.fbs)

--]]

local LongEnum = {
  LongOne = 2,
  LongTwo = 4,
  LongBig = 1099511627776,
}

return LongEnum