// automatically generated by the FlatBuffers compiler, do not modify

/* eslint-disable @typescript-eslint/no-unused-vars, @typescript-eslint/no-explicit-any, @typescript-eslint/no-non-null-assertion */

/**
 * New schema language features that are not supported by old code generators.
 */
export enum AdvancedFeatures {
  AdvancedArrayFeatures = '1',
  AdvancedUnionFeatures = '2',
  OptionalScalars = '4',
  DefaultVectorsAndStrings = '8',
}
