// automatically generated by the FlatBuffers compiler, do not modify
/* eslint-disable @typescript-eslint/no-unused-vars, @typescript-eslint/no-explicit-any,
 * @typescript-eslint/no-non-null-assertion */
import {Monster} from '../../my-game/example/monster.js';
export var AnyAmbiguousAliases;
(function(AnyAmbiguousAliases) {
AnyAmbiguousAliases[AnyAmbiguousAliases['NONE'] = 0] = 'NONE';
AnyAmbiguousAliases[AnyAmbiguousAliases['M1'] = 1] = 'M1';
AnyAmbiguousAliases[AnyAmbiguousAliases['M2'] = 2] = 'M2';
AnyAmbiguousAliases[AnyAmbiguousAliases['M3'] = 3] = 'M3';
})(AnyAmbiguousAliases || (AnyAmbiguousAliases = {}));
export function unionToAnyAmbiguousAliases(type, accessor) {
  switch (AnyAmbiguousAliases[type]) {
    case 'NONE':
      return null;
    case 'M1':
      return accessor(new Monster());
    case 'M2':
      return accessor(new Monster());
    case 'M3':
      return accessor(new Monster());
    default:
      return null;
  }
}
export function unionListToAnyAmbiguousAliases(type, accessor, index) {
  switch (AnyAmbiguousAliases[type]) {
    case 'NONE':
      return null;
    case 'M1':
      return accessor(index, new Monster());
    case 'M2':
      return accessor(index, new Monster());
    case 'M3':
      return accessor(index, new Monster());
    default:
      return null;
  }
}
