#[ MyGame.Example.Race
  Automatically generated by the FlatBuffers compiler, do not modify.
  Or modify. I'm a message, not a cop.

  flatc version: 25.9.23

  Declared by  : 
  Rooting type : MyGame.Example.Monster ()
]#

type Race*{.pure.} = enum
  None = -1.int8,
  Human = 0.int8,
  Dwarf = 1.int8,
  Elf = 2.int8,
