// automatically generated by the FlatBuffers compiler, do not modify

package NamespaceA.NamespaceB;

import com.google.flatbuffers.*;
import java.nio.*;
import java.util.*;

public class TableInNestedNST {
  private int foo;

  public int getFoo() {
    return foo;
  }

  public void setFoo(int foo) {
    this.foo = foo;
  }

  public TableInNestedNST() {
    this.foo = 0;
  }
}
