// automatically generated by the FlatBuffers compiler, do not modify

package MyGame.Example;

@SuppressWarnings("unused")
public final class AnyUniqueAliases {
  private AnyUniqueAliases() { }
  public static final byte NONE = 0;
  public static final byte M = 1;
  public static final byte TS = 2;
  public static final byte M2 = 3;

  public static final String[] names = { "NONE", "M", "TS", "M2", };

  public static String name(int e) { return names[e]; }
}

