// This file should contain the basics of flatbuffers that all languages should
// support.

namespace flatbuffers.goldens;

table Galaxy {
  num_stars:long;
}

table Universe {
  age:double;
  galaxies:[Galaxy];
}

root_type Universe;