// automatically generated by the FlatBuffers compiler, do not modify
/* eslint-disable @typescript-eslint/no-unused-vars, @typescript-eslint/no-explicit-any,
 * @typescript-eslint/no-non-null-assertion */
export var LongEnum;
(function(LongEnum) {
LongEnum['LongOne'] = '2';
LongEnum['LongTwo'] = '4';
LongEnum['LongBig'] = '1099511627776';
})(LongEnum || (LongEnum = {}));
