// <auto-generated>
//  automatically generated by the FlatBuffers compiler, do not modify
// </auto-generated>

namespace optional_scalars
{

public enum OptionalByte : sbyte
{
  None = 0,
  One = 1,
  Two = 2,
};


}
