<?php
// automatically generated by the FlatBuffers compiler, do not modify

namespace MyGame\Example;

class Race
{
    const None = -1;
    const Human = 0;
    const Dwarf = 1;
    const Elf = 2;

    private static $names = array(
        Race::None=>"None",
        Race::Human=>"Human",
        Race::Dwarf=>"Dwarf",
        Race::Elf=>"Elf",
    );

    public static function Name($e)
    {
        if (!isset(self::$names[$e])) {
            throw new \Exception();
        }
        return self::$names[$e];
    }
}
