// <auto-generated>
//  automatically generated by the FlatBuffers compiler, do not modify
// </auto-generated>

namespace MyGame.Example
{

[Newtonsoft.Json.JsonConverter(typeof(Newtonsoft.Json.Converters.StringEnumConverter))]
public enum Race : sbyte
{
  None = -1,
  Human = 0,
  Dwarf = 1,
  Elf = 2,
};


}
