#[ MyGame.Example.Color
  Automatically generated by the FlatBuffers compiler, do not modify.
  Or modify. I'm a message, not a cop.

  flatc version: 25.9.23

  Declared by  : 
  Rooting type : MyGame.Example.Monster ()
]#

#  Composite components of Monster color.
type Color*{.pure.} = enum
  Red = 1.uint8,
  #  \brief color Green
  #  Green is bit_flag with value (1u << 1)
  Green = 2.uint8,
  #  \brief color Blue (1u << 3)
  Blue = 8.uint8,
